==> network_manager-0.0.9.tar.gz: lib/network_manager.dart <==
//import 'dart:async';
//import 'dart:convert';
//import 'dart:core';
//import 'dart:core' as prefix0;
//import 'dart:io';
//import 'package:flightreport/global/MainModel.dart';
//import 'package:flightreport/widgets/MyAlert.dart';
//import 'package:flutter/services.dart';
//
//import 'package:flutter/material.dart';
//import 'package:http/http.dart' as http;
//import 'package:scoped_model/scoped_model.dart';
//
//import 'ConstStrings.dart';
//const defaultHedaer = {'Content-Type': 'application/json'};
//
//class NetworkManager {
//  String api;
//  Map<String, dynamic> req;
//  Function onSuccess;
//  Function onFailed;
//  Function onStart;
//  Function onEnd;
//  BuildContext context;
//  bool isAuto;
//  dynamic headers;
//
//  NetworkManager({
//    @required this.api,
//    @required this.req,
//    @required this.onSuccess,
//    @required this.onFailed,
//    @required this.onStart,
//    @required this.onEnd,
//    @required this.context,
//    this.headers = defaultHedaer,
//    this.isAuto = false
//  });
//
//  void _request() {
//    MainModel model = ScopedModel.of(context);
//    onStart();
//    print("REQ:"+req.toString());
//    print("API:"+(api.contains("http")?api:model.selectedServer.address+api));
//    http
//        .post(
//      api.contains("http")?api:model.selectedServer.address+api,
//      headers: {'Content-Type': 'application/json',
//                "UserId": json.encode(model.user==null?null:model.user.userId),
//                "UserKey": null,
//                "Token": model.user==null?json.encode(null):model.user.token,
//                'Source': json.encode(true)
//      },
//      body: json.encode(req),
//    )
//        .then((response) {
//      //ToDo check result coee
//      var resultData = json.decode(response.body);
//      print(resultData.toString());
//      if (resultData["IsSuccess"]==null || resultData["IsSuccess"]==true ) {
//        if((resultData["ResultCode"]==null||resultData["ResultCode"]>0)){
//          onSuccess(resultData);
//        }
//      } else {
//        onFailed(resultData);
//        if(!isAuto){
//          showDialog(
//            barrierDismissible: false,
//            context: context,
//            builder: (BuildContext context) => MyAlert(
//              errorText: resultData["ErrorMessage"]==null?"":resultData["ErrorMessage"],
//              buttonText: constStrings(model.appLang, "TryAgain"),
//            ),
//          ).then((value){
//            if(value!=null){request();}
//          });
//        }
//
//      }
//
//      onEnd();
//    }).catchError((error) {
//      if (error.toString().contains("SocketException")) {
//        print("------------------_+++++++++++");
//        if(!isAuto){
//          showDialog(
//            barrierDismissible: false,
//            context: context,
//            builder: (BuildContext context) => MyAlert(
//              errorText: "Connection Intrupted",
//              buttonText: constStrings(model.appLang, "TryAgain"),
//
//            ),
//          ).then((value){
//            if(value!=null){request();}
//          });
//        }
//      }
//      onFailed(error.toString());
//      onEnd();
//    }).timeout(Duration(seconds: 15), onTimeout: () {
//      print("Untruppted");
//      dynamic response = {
//        "ResultCode": "-111111111",
//        "ResultText": constStrings(model.appLang, "ConnectionIntrupted")
//      };
//      model.setGlobalLoading(false);
//      onFailed(response);
//      if(!isAuto){
//        showDialog(
//          barrierDismissible: false,
//          context: context,
//          builder: (BuildContext context) => MyAlert(
//            errorText: "Connection Intrupted",
//            buttonText: constStrings(model.appLang, "TryAgain"),
//
//          ),
//        ).then((value){
//          if(value!=null){request();}
//        });
//      }
//    });
//  }
//
//  Function get request {
//    return _request;
//  }
//}

import 'dart:convert';
import 'dart:ffi';

import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:handy_ui_kit/handy_ui_kit.dart';

/// A Network Manager.
class NetworkManagerMetaData {
  /// have several http functions

  ///called on start every request
  Function onStartDefault = () {};

  ///called on End every request
  Function onEndDefault = () {};

  ///called on Success every request
  Function onSuccessDefault = () {};

  ///called on Failed every request
  Function onFailedDefault = () {};

  ///String token which is used in header
  String token = "";

  ///other rules for check beside result code 200 should be a function return bool value
  bool Function(dynamic responseJson) successRules = (dynamic responseJson) => true;

  BaseOptions dioOption = Dio().options;

  /// a function to extract error as string
  String Function(dynamic error) errorMsgExtractor = (error) => error.toString();
  String Function(dynamic msg) successMsgExtractor = (msg) => msg.toString();

  static final NetworkManagerMetaData _singleton = NetworkManagerMetaData._internal();

  NetworkManagerMetaData._internal();

  factory NetworkManagerMetaData() {
    return _singleton;
  }

  static setAccessToken(String token) {
    _singleton.token = token;
  }

  static initialize({
    String baseUrl,
    int timeout = 3000,
    Function onStartDefault,
    Function onEndDefault,
    Function onSuccessDefault,
    Function onFailedDefault,
    Map<String, dynamic> headers,
    bool Function(dynamic responseJson) successRules,
    String Function(dynamic error) errorMsgExtractor,
    String Function(dynamic msg) successMsgExtractor,
  }) {
    if (onSuccessDefault != null) {
      _singleton.onStartDefault = onStartDefault;
    }
    if (onEndDefault != null) {
      _singleton.onEndDefault = onEndDefault;
    }
    if (onSuccessDefault != null) {
      _singleton.onSuccessDefault = onSuccessDefault;
    }
    if (onFailedDefault != null) {
      _singleton.onFailedDefault = onFailedDefault;
    }
    if (successRules != null) {
      _singleton.successRules = successRules;
    }
    if (baseUrl != null) {
      _singleton.dioOption.baseUrl = baseUrl;
    }
    if (headers != null) {
      _singleton.dioOption.headers = headers;
    }
    if (timeout != null) {
      _singleton.dioOption.connectTimeout = timeout;
      _singleton.dioOption.sendTimeout = timeout;
      _singleton.dioOption.receiveTimeout = timeout;
    }

    ///You can set LogInterceptor to print request/response log automaticlly
//    _singleton.dio.interceptors.add(LogInterceptor(responseBody: false));
    if (errorMsgExtractor != null) {
      _singleton.errorMsgExtractor = errorMsgExtractor;
    }
    if (successMsgExtractor != null) {
      _singleton.successMsgExtractor = errorMsgExtractor;
    }
  }
}

class NetworkManager {
  ///need context to show dialog or pop and push
  BuildContext context;

  ///called on start this instance of Network Manager request
  Function onStart = () {};

  ///called on end this instance of Network Manager request
  Function onEnd = () {};

  ///called on success this instance of Network Manager request
  void Function(BuildContext context, dynamic responseJson) onSuccess = (BuildContext context, dynamic responseJson) {};

  ///called on failed this instance of Network Manager request
  void Function(BuildContext context, dynamic responseJson) onFailed = (BuildContext context, dynamic responseJson) {};

  ///check to show dialog when when request is success
  bool showFailedDialog = false;

  ///check to show dialog when when request is failed
  bool showSuccessDialog = false;

  ///check to call default functions each step
  bool callDefaults = true;

  ///api address you're requesting
  String api;

  ///Map<String dynamic> request data for post methods
  Map<String, dynamic> req;

  ///Map<String dynamic> query data for get methods
  Map<String, dynamic> query;

  /// dio instance for making requests
  Dio dio = Dio()..options = BaseOptions(headers: {'Content-Type': 'application/json'});

  Function onStartDefault = () {};

  ///called on End every request
  Function onEndDefault = () {};

  ///called on Success every request
  Function onSuccessDefault = () {};

  ///called on Failed every request
  Function onFailedDefault = () {};

  ///String token which is used in header
  String token = "";

  ///other rules for check beside result code 200 should be a function return bool value
  bool Function(dynamic responseJson) successRules = (dynamic responseJson) => true;

  BaseOptions dioOption = Dio().options;

  /// a function to extract error as string
  String Function(dynamic error) errorMsgExtractor = (error) => error.toString();
  String Function(dynamic msg) successMsgExtractor = (msg) => msg.toString();

  NetworkManager._(
      {this.showSuccessDialog,
        this.callDefaults,
        this.onStartDefault,
        this.onEndDefault,
        this.onFailedDefault,
        this.successRules,
        this.successMsgExtractor,
        this.onSuccessDefault,
        this.dio,
        this.errorMsgExtractor,
        this.showFailedDialog,
        this.token,
        this.req,
        this.context,
        this.api,
        this.onSuccess,
        this.onEnd,
        this.onFailed,
        this.query,
        this.onStart});

  factory NetworkManager.instance(
      {@required BuildContext context,
        @required String api,
        Map<String, dynamic> req,
        Map<String, dynamic> query,
        void Function(BuildContext context, dynamic result) onSuccess,
        void Function(BuildContext context, dynamic result) onFailed,
        Function onStart,
        Function onEnd,
        bool showFailedDialog = false,
        bool callDefaults = true,
        bool showSuccessDialog = false}) {
    NetworkManagerMetaData meta = NetworkManagerMetaData();
    return NetworkManager._(
      context: context,
      api: api,
      req: req,
      query: query,
      onSuccess: onSuccess??(BuildContext context, dynamic responseJson) {},
      onFailed: onFailed??(BuildContext context, dynamic responseJson) {},
      onStart: onStart??meta.onStartDefault,
      onEnd: onEnd??meta.onEndDefault,
      showFailedDialog: showFailedDialog,
      callDefaults: callDefaults,
      showSuccessDialog: showSuccessDialog,
      onSuccessDefault: meta.onSuccessDefault,
      onFailedDefault: meta.onFailedDefault,
      onStartDefault: meta.onStartDefault,
      onEndDefault: meta.onEndDefault,
      token: meta.token,
      successRules: meta.successRules,
      successMsgExtractor: meta.successMsgExtractor,
      errorMsgExtractor: meta.errorMsgExtractor,
      dio: Dio()..options = meta.dioOption,
    );
  }

  _requestPost() {
    if (callDefaults) {
      onStartDefault();
    }
    onStart();
    print("Posting $req in $api");
    dio.post(api, data: req).then((response) {
      var statusCode = response.statusCode;
      var resultJson;
      if(response.data.runtimeType == String){
        resultJson = jsonDecode(response.data);
      }else{
        resultJson = Map<String, dynamic>.from(response.data);
      }
      print("***Status Code*** : $statusCode");
      print("***Response Data*** : $resultJson");
      if (statusCode == 200) {
        if (successRules(resultJson)) {
          if (callDefaults) {
            onSuccessDefault();
          }
          onSuccess(context, resultJson);
          if (showSuccessDialog) {
            showDialog(
                context: context,
                builder: (context) => MyDialog(
                  title: "Success Operation",
                  body: successMsgExtractor(resultJson),
                  type: DialogType.success,
                ));
          }
        } else {
          if (callDefaults) {
            onFailedDefault();
          }
          onFailed(context, errorMsgExtractor(resultJson));
          if (showFailedDialog) {
            showDialog(
                context: context,
                builder: (context) => MyDialog(
                  title: "Failed Operation",
                  body: errorMsgExtractor(resultJson),
                  type: DialogType.warning,
                  confirmText: "Retry",
                )).then((value) {
              if (value != null) {
                _requestPost();
              }
            });
          }
        }
      } else {
        if (callDefaults) {
          onFailedDefault();
        }
        onFailed(context, resultJson);
        if (showFailedDialog) {
          showDialog(
              context: context,
              builder: (context) => MyDialog(
                title: "($statusCode) Connection Interrupted",
                body: resultJson.toString(),
                type: DialogType.error,
                confirmText: "Retry",
              )).then((value) {
            if (value != null) {
              _requestPost();
            }
          });
        }
      }
      if (callDefaults) {
        onEndDefault();
      }
      onEnd();
    }).catchError((e) {
      if(e is DioError){
        DioError ee = e;
        print("Error: ${ee.error}");
        if (callDefaults) {
          onFailedDefault();
          onEndDefault();
        }
        onFailed(context, ee.error);
        onEnd();
        if (showFailedDialog) {
          showDialog(
              context: context,
              builder: (context) => MyDialog(
                title: "Failed Operation",
                body: ee.error.toString(),
                type: DialogType.error,
                confirmText: "Retry",
              )).then((value) {
            if (value != null) {
              _requestPost();
            }
          });
        }
      }else{
        print("Error: $e");
        if (callDefaults) {
          onFailedDefault();
          onEndDefault();
        }
        onFailed(context, e);
        onEnd();
        if (showFailedDialog) {
          showDialog(
              context: context,
              builder: (context) => MyDialog(
                title: "Failed Operation",
                body: e.toString(),
                type: DialogType.error,
                confirmText: "Retry",
              )).then((value) {
            if (value != null) {
              _requestPost();
            }
          });
        }
      }

    });
  }

  _requestGet() {
    if (callDefaults) {
      onStartDefault();
    }
    onStart();
    print("Getting $api with $query");
    dio.get(api, queryParameters: req).then((response) {
      var statusCode = response.statusCode;
      var resultJson = response.data;
      print("***Status Code*** : $statusCode");
      print("***Response Data*** : $resultJson");
      if (statusCode == 200 && successRules(resultJson)) {
        if (callDefaults) {
          onSuccessDefault();
        }
        onSuccess(context, resultJson);
      } else {
        if (callDefaults) {
          onFailedDefault();
        }
        onFailed(context, resultJson);
      }
      if (callDefaults) {
        onEndDefault();
      }
      onEnd();
    }).catchError((e) {
      if (callDefaults) {
        onFailedDefault();
        onEndDefault();
      }
      print("Error: $e");
      onFailed(context, e);
      onEnd();
    });
  }

  get requestPost => _requestPost;

  get requestGet => _requestGet;
}

==> network_manager-0.0.10.tar.gz: lib/network_manager.dart <==
//import 'dart:async';
//import 'dart:convert';
//import 'dart:core';
//import 'dart:core' as prefix0;
//import 'dart:io';
//import 'package:flightreport/global/MainModel.dart';
//import 'package:flightreport/widgets/MyAlert.dart';
//import 'package:flutter/services.dart';
//
//import 'package:flutter/material.dart';
//import 'package:http/http.dart' as http;
//import 'package:scoped_model/scoped_model.dart';
//
//import 'ConstStrings.dart';
//const defaultHedaer = {'Content-Type': 'application/json'};
//
//class NetworkManager {
//  String api;
//  Map<String, dynamic> req;
//  Function onSuccess;
//  Function onFailed;
//  Function onStart;
//  Function onEnd;
//  BuildContext context;
//  bool isAuto;
//  dynamic headers;
//
//  NetworkManager({
//    @required this.api,
//    @required this.req,
//    @required this.onSuccess,
//    @required this.onFailed,
//    @required this.onStart,
//    @required this.onEnd,
//    @required this.context,
//    this.headers = defaultHedaer,
//    this.isAuto = false
//  });
//
//  void _request() {
//    MainModel model = ScopedModel.of(context);
//    onStart();
//    print("REQ:"+req.toString());
//    print("API:"+(api.contains("http")?api:model.selectedServer.address+api));
//    http
//        .post(
//      api.contains("http")?api:model.selectedServer.address+api,
//      headers: {'Content-Type': 'application/json',
//                "UserId": json.encode(model.user==null?null:model.user.userId),
//                "UserKey": null,
//                "Token": model.user==null?json.encode(null):model.user.token,
//                'Source': json.encode(true)
//      },
//      body: json.encode(req),
//    )
//        .then((response) {
//      //ToDo check result coee
//      var resultData = json.decode(response.body);
//      print(resultData.toString());
//      if (resultData["IsSuccess"]==null || resultData["IsSuccess"]==true ) {
//        if((resultData["ResultCode"]==null||resultData["ResultCode"]>0)){
//          onSuccess(resultData);
//        }
//      } else {
//        onFailed(resultData);
//        if(!isAuto){
//          showDialog(
//            barrierDismissible: false,
//            context: context,
//            builder: (BuildContext context) => MyAlert(
//              errorText: resultData["ErrorMessage"]==null?"":resultData["ErrorMessage"],
//              buttonText: constStrings(model.appLang, "TryAgain"),
//            ),
//          ).then((value){
//            if(value!=null){request();}
//          });
//        }
//
//      }
//
//      onEnd();
//    }).catchError((error) {
//      if (error.toString().contains("SocketException")) {
//        print("------------------_+++++++++++");
//        if(!isAuto){
//          showDialog(
//            barrierDismissible: false,
//            context: context,
//            builder: (BuildContext context) => MyAlert(
//              errorText: "Connection Intrupted",
//              buttonText: constStrings(model.appLang, "TryAgain"),
//
//            ),
//          ).then((value){
//            if(value!=null){request();}
//          });
//        }
//      }
//      onFailed(error.toString());
//      onEnd();
//    }).timeout(Duration(seconds: 15), onTimeout: () {
//      print("Untruppted");
//      dynamic response = {
//        "ResultCode": "-111111111",
//        "ResultText": constStrings(model.appLang, "ConnectionIntrupted")
//      };
//      model.setGlobalLoading(false);
//      onFailed(response);
//      if(!isAuto){
//        showDialog(
//          barrierDismissible: false,
//          context: context,
//          builder: (BuildContext context) => MyAlert(
//            errorText: "Connection Intrupted",
//            buttonText: constStrings(model.appLang, "TryAgain"),
//
//          ),
//        ).then((value){
//          if(value!=null){request();}
//        });
//      }
//    });
//  }
//
//  Function get request {
//    return _request;
//  }
//}

import 'dart:convert';
import 'dart:ffi';

import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:handy_ui_kit/handy_ui_kit.dart';

/// A Network Manager.
class NetworkManagerMetaData {
  /// have several http functions

  ///called on start every request
  Function onStartDefault = () {};

  ///called on End every request
  Function onEndDefault = () {};

  ///called on Success every request
  Function onSuccessDefault = () {};

  ///called on Failed every request
  Function onFailedDefault = () {};

  ///String token which is used in header
  String token = "";

  ///other rules for check beside result code 200 should be a function return bool value
  bool Function(dynamic responseJson) successRules = (dynamic responseJson) => true;

  BaseOptions dioOption = Dio().options;

  /// a function to extract error as string
  String Function(dynamic error) errorMsgExtractor = (error) => error.toString();
  String Function(dynamic msg) successMsgExtractor = (msg) => msg.toString();

  static final NetworkManagerMetaData _singleton = NetworkManagerMetaData._internal();

  NetworkManagerMetaData._internal();

  factory NetworkManagerMetaData() {
    return _singleton;
  }

  static setAccessToken(String token) {
    _singleton.token = token;
  }

  static initialize({
    String baseUrl,
    int timeout = 3000,
    Function onStartDefault,
    Function onEndDefault,
    Function onSuccessDefault,
    Function onFailedDefault,
    Map<String, dynamic> headers,
    bool Function(dynamic responseJson) successRules,
    String Function(dynamic error) errorMsgExtractor,
    String Function(dynamic msg) successMsgExtractor,
  }) {
    if (onSuccessDefault != null) {
      _singleton.onStartDefault = onStartDefault;
    }
    if (onEndDefault != null) {
      _singleton.onEndDefault = onEndDefault;
    }
    if (onSuccessDefault != null) {
      _singleton.onSuccessDefault = onSuccessDefault;
    }
    if (onFailedDefault != null) {
      _singleton.onFailedDefault = onFailedDefault;
    }
    if (successRules != null) {
      _singleton.successRules = successRules;
    }
    if (baseUrl != null) {
      _singleton.dioOption.baseUrl = baseUrl;
    }
    if (headers != null) {
      _singleton.dioOption.headers = headers;
    }
    if (timeout != null) {
      _singleton.dioOption.connectTimeout = timeout;
      _singleton.dioOption.sendTimeout = timeout;
      _singleton.dioOption.receiveTimeout = timeout;
    }

    ///You can set LogInterceptor to print request/response log automaticlly
//    _singleton.dio.interceptors.add(LogInterceptor(responseBody: false));
    if (errorMsgExtractor != null) {
      _singleton.errorMsgExtractor = errorMsgExtractor;
    }
    if (successMsgExtractor != null) {
      _singleton.successMsgExtractor = errorMsgExtractor;
    }
  }
}

class NetworkManager {
  ///need context to show dialog or pop and push
  BuildContext context;

  ///called on start this instance of Network Manager request
  Function onStart = () {};

  ///called on end this instance of Network Manager request
  Function onEnd = () {};

  ///called on success this instance of Network Manager request
  void Function(BuildContext context, dynamic responseJson) onSuccess = (BuildContext context, dynamic responseJson) {};

  ///called on failed this instance of Network Manager request
  void Function(BuildContext context, dynamic responseJson) onFailed = (BuildContext context, dynamic responseJson) {};

  ///check to show dialog when when request is success
  bool showFailedDialog = false;

  ///check to show dialog when when request is failed
  bool showSuccessDialog = false;

  ///check to call default functions each step
  bool callDefaults = true;

  ///api address you're requesting
  String api;

  ///Map<String dynamic> request data for post methods
  Map<String, dynamic> req;

  ///Map<String dynamic> query data for get methods
  Map<String, dynamic> query;

  /// dio instance for making requests
  Dio dio = Dio()..options = BaseOptions(headers: {'Content-Type': 'application/json'});

  Function onStartDefault = () {};

  ///called on End every request
  Function onEndDefault = () {};

  ///called on Success every request
  Function onSuccessDefault = () {};

  ///called on Failed every request
  Function onFailedDefault = () {};

  ///String token which is used in header
  String token = "";

  ///other rules for check beside result code 200 should be a function return bool value
  bool Function(dynamic responseJson) successRules = (dynamic responseJson) => true;

  BaseOptions dioOption = Dio().options;

  /// a function to extract error as string
  String Function(dynamic error) errorMsgExtractor = (error) => error.toString();
  String Function(dynamic msg) successMsgExtractor = (msg) => msg.toString();

  CancelToken cancelToken;

  NetworkManager._(
      {this.showSuccessDialog,
        this.callDefaults,
        this.onStartDefault,
        this.onEndDefault,
        this.onFailedDefault,
        this.successRules,
        this.successMsgExtractor,
        this.onSuccessDefault,
        this.dio,
        this.errorMsgExtractor,
        this.showFailedDialog,
        this.token,
        this.req,
        this.context,
        this.api,
        this.onSuccess,
        this.onEnd,
        this.onFailed,
        this.query,
        this.onStart,
        this.cancelToken,
      });

  factory NetworkManager.instance(
      {@required BuildContext context,
        @required String api,
        Map<String, dynamic> req,
        Map<String, dynamic> query,
        void Function(BuildContext context, dynamic result) onSuccess,
        void Function(BuildContext context, dynamic result) onFailed,
        Function onStart,
        Function onEnd,
        CancelToken cancelToken,
        bool showFailedDialog = false,
        bool callDefaults = true,
        bool showSuccessDialog = false}) {
    NetworkManagerMetaData meta = NetworkManagerMetaData();
    return NetworkManager._(
      context: context,
      api: api,
      req: req,
      query: query,
      onSuccess: onSuccess??(BuildContext context, dynamic responseJson) {},
      onFailed: onFailed??(BuildContext context, dynamic responseJson) {},
      onStart: onStart??meta.onStartDefault,
      onEnd: onEnd??meta.onEndDefault,
      cancelToken: cancelToken,
      showFailedDialog: showFailedDialog,
      callDefaults: callDefaults,
      showSuccessDialog: showSuccessDialog,
      onSuccessDefault: meta.onSuccessDefault,
      onFailedDefault: meta.onFailedDefault,
      onStartDefault: meta.onStartDefault,
      onEndDefault: meta.onEndDefault,
      token: meta.token,
      successRules: meta.successRules,
      successMsgExtractor: meta.successMsgExtractor,
      errorMsgExtractor: meta.errorMsgExtractor,
      dio: Dio()..options = meta.dioOption,
    );
  }

  _requestPost() {
    if (callDefaults) {
      onStartDefault();
    }
    onStart();
    print("Posting $req in $api");
    dio.post(api, data: req,cancelToken: cancelToken).then((response) {
      var statusCode = response.statusCode;
      var resultJson;
      if(response.data.runtimeType == String){
        resultJson = jsonDecode(response.data);
      }else{
        resultJson = Map<String, dynamic>.from(response.data);
      }
      print("***Status Code*** : $statusCode");
      print("***Response Data*** : $resultJson");
      if (statusCode == 200) {
        if (successRules(resultJson)) {
          if (callDefaults) {
            onSuccessDefault();
          }
          onSuccess(context, resultJson);
          if (showSuccessDialog) {
            showDialog(
                context: context,
                builder: (context) => MyDialog(
                  title: "Success Operation",
                  body: successMsgExtractor(resultJson),
                  type: DialogType.success,
                ));
          }
        } else {
          if (callDefaults) {
            onFailedDefault();
          }
          onFailed(context, errorMsgExtractor(resultJson));
          if (showFailedDialog) {
            showDialog(
                context: context,
                builder: (context) => MyDialog(
                  title: "Failed Operation",
                  body: errorMsgExtractor(resultJson),
                  type: DialogType.warning,
                  confirmText: "Retry",
                )).then((value) {
              if (value != null) {
                _requestPost();
              }
            });
          }
        }
      } else {
        if (callDefaults) {
          onFailedDefault();
        }
        onFailed(context, resultJson);
        if (showFailedDialog) {
          showDialog(
              context: context,
              builder: (context) => MyDialog(
                title: "($statusCode) Connection Interrupted",
                body: resultJson.toString(),
                type: DialogType.error,
                confirmText: "Retry",
              )).then((value) {
            if (value != null) {
              _requestPost();
            }
          });
        }
      }
      if (callDefaults) {
        onEndDefault();
      }
      onEnd();
    }).catchError((e) {
      if(e is DioError){
        DioError ee = e;
        print("Error: ${ee.error}");
        if (callDefaults) {
          onFailedDefault();
          onEndDefault();
        }
        onFailed(context, ee.error);
        onEnd();
        if (showFailedDialog) {
          showDialog(
              context: context,
              builder: (context) => MyDialog(
                title: "Failed Operation",
                body: ee.error.toString(),
                type: DialogType.error,
                confirmText: "Retry",
              )).then((value) {
            if (value != null) {
              _requestPost();
            }
          });
        }
      }else{
        print("Error: $e");
        if (callDefaults) {
          onFailedDefault();
          onEndDefault();
        }
        onFailed(context, e);
        onEnd();
        if (showFailedDialog) {
          showDialog(
              context: context,
              builder: (context) => MyDialog(
                title: "Failed Operation",
                body: e.toString(),
                type: DialogType.error,
                confirmText: "Retry",
              )).then((value) {
            if (value != null) {
              _requestPost();
            }
          });
        }
      }

    });
  }

  _requestGet() {
    if (callDefaults) {
      onStartDefault();
    }
    onStart();
    print("Getting $api with $query");
    dio.get(api, queryParameters: req).then((response) {
      var statusCode = response.statusCode;
      var resultJson = response.data;
      print("***Status Code*** : $statusCode");
      print("***Response Data*** : $resultJson");
      if (statusCode == 200 && successRules(resultJson)) {
        if (callDefaults) {
          onSuccessDefault();
        }
        onSuccess(context, resultJson);
      } else {
        if (callDefaults) {
          onFailedDefault();
        }
        onFailed(context, resultJson);
      }
      if (callDefaults) {
        onEndDefault();
      }
      onEnd();
    }).catchError((e) {
      if (callDefaults) {
        onFailedDefault();
        onEndDefault();
      }
      print("Error: $e");
      onFailed(context, e);
      onEnd();
    });
  }

  get requestPost => _requestPost;

  get requestGet => _requestGet;
}
